@@ -55,6 +55,143 @@ iterateOmegaGrid <- function(vData, omegasIter, betaRestrictions, timePoints = s
 
 }
 
+################################################################################
+# Internal function: returns the parallelized apply function depending on the OS.
+# Returns the apply function to be used.
+################################################################################
+getApply <- function(parallelize = FALSE){
+
+  getApply_Rbase <- function(){
+    usedApply <- function(FUN, X, ...) t(apply(X = X, MARGIN = 1, FUN = FUN, ...))
+  }
+
+  getParallelApply_Windows <- function(parallelCluster){
+    usedApply <- function(FUN, X, ...) t(parallel::parApply(parallelCluster, FUN = FUN,
+                                                            X = X, MARGIN = 1, ...))
+    return(usedApply)
+  }
+
+  parallelFunction_Unix<-function(nCores){
+    # A parallelized apply function does not exist, so it must be translated to a lapply
+    usedApply <- function(FUN, X, ...){
+      matrix(unlist(parallel::mclapply(X = asplit(X, 1), FUN = FUN, mc.cores = nCores, ...)),
+             nrow = nrow(X), byrow = T)
+    }
+    return(usedApply)
+  }
+
+  nCores <- min(12, parallel::detectCores() - 1)
+
+  if(parallelize){
+    # different ways to implement parallelization depending on OS:
+    if(.Platform$OS.type == "windows"){
+      parallelCluster <- parallel::makePSOCKcluster(nCores)
+      doParallel::registerDoParallel(parallelCluster)
+      usedApply <- getParallelApply_Windows(parallelCluster)
+    }else{
+      usedApply <- parallelFunction_Unix(nCores)
+      parallelCluster <- NULL
+    }
+  }else{
+    # R base apply:
+    usedApply <- getApply_Rbase()
+    parallelCluster <- NULL
+  }
+
+  return(list(usedApply, parallelCluster))
+}
+
+################################################################################
+# Internal function: to estimate M, A and beta initial parameters
+# also returns residual sum of squared (RSS).
+# Arguments:
+#    alphaOmegaParameters: vector with the fixed values of alpha and omega
+#    vData: data to be fitted an FMM model.
+#    timePoints: one single period time points.
+# Returns a 6-length numerical vector: M, A, alpha, beta, omega and RSS
+################################################################################
+
+step1FMMOld <- function(alphaOmegaParameters, vData, timePoints) {
+
+  alphaParameter <- alphaOmegaParameters[1]
+  omegaParameter <- alphaOmegaParameters[2]
+
+  mobiusTerm <- 2*atan(omegaParameter*tan((timePoints - alphaParameter)/2))
+  tStar <- alphaParameter + mobiusTerm
+
+  # Given alpha and omega, a cosinor model is computed with t* in
+  # order to obtain delta (cosCoeff) and gamma (sinCoeff).
+  # Linear Model exact expressions are used to improve performance.
+  costStar <- cos(tStar)
+  sentstar <- sin(tStar)
+  covMatrix <- stats::cov(cbind(vData, costStar, sentstar))
+  denominator <- covMatrix[2,2]*covMatrix[3,3] - covMatrix[2,3]^2
+  cosCoeff <- (covMatrix[1,2]*covMatrix[3,3] -
+                 covMatrix[1,3]*covMatrix[2,3])/denominator
+  sinCoeff <- (covMatrix[1,3]*covMatrix[2,2] -
+                 covMatrix[1,2]*covMatrix[2,3])/denominator
+  mParameter <- mean(vData) - cosCoeff*mean(costStar) - sinCoeff*mean(sentstar)
+
+  phiEst <- atan2(-sinCoeff, cosCoeff)
+  aParameter <- sqrt(cosCoeff^2 + sinCoeff^2)
+  betaParameter <- (phiEst+alphaParameter)%%(2*pi)
+
+  mobiusRegression <- mParameter + aParameter*cos(betaParameter + mobiusTerm)
+  residualSS <- sum((vData - mobiusRegression)^2)/length(timePoints)
+
+  return(c(mParameter, aParameter, alphaParameter, betaParameter,
+           omegaParameter, residualSS))
+}
+
+################################################################################
+# Internal function: to find the optimal initial parameter estimation
+# Arguments:
+#    vData: data to be fitted an FMM model.
+#    step1: a data.frame with estimates of
+#           M, A, alpha, beta, omega, RSS as columns.
+# Returns the optimal row of step1 argument.
+# optimum: minimum RSS with several stability conditions.
+################################################################################
+bestStep1 <- function(vData, step1){
+
+  # step1 in decreasing order by RSS
+  orderedModelParameters <- order(step1[,"RSS"])
+
+  maxVData <- max(vData)
+  minVData <- min(vData)
+  nObs <- length(vData)
+
+  # iterative search: go through rows ordered step 1
+  #    until the first one that verifies the stability conditions
+  bestModelFound <- FALSE
+  i <- 1
+  while(!bestModelFound){
+    # parameters
+    mParameter <- step1[orderedModelParameters[i], "M"]
+    aParameter <- step1[orderedModelParameters[i], "A"]
+    sigma <- sqrt(step1[orderedModelParameters[i], "RSS"]*nObs/(nObs-5))
+
+    # stability conditions
+    amplitudeUpperBound <- mParameter + aParameter
+    amplitudeLowerBound <- mParameter - aParameter
+    rest1 <- amplitudeUpperBound <= maxVData + 1.96*sigma
+    rest2 <- amplitudeLowerBound >= minVData - 1.96*sigma
+
+    # it is necessary to check that there are no NA,
+    # because it can be an extreme solution
+    if(is.na(rest1)) rest1 <- FALSE
+    if(is.na(rest2)) rest2 <- FALSE
+
+    if(rest1 & rest2){
+      bestModelFound <- TRUE
+    } else {
+      i <- i+1
+    }
+    if(i > nrow(step1))
+      return(NULL)
+  }
+  return(step1[orderedModelParameters[i],])
+}
 
 #######################################################################################
 # Internal function: backfitting algorithm with fixed omegas.
@@ -90,7 +227,7 @@ backfittingRestr <- function(vData, omegas, nback, betaRestrictions,
       backFittingData <- vData - apply(as.matrix(fittedValuesPerComponent[,-j]), 1, sum)
       # component j fitting using fitFMM_unit_restr function
       fittedFMMPerComponent[[j]] <- fitFMM_unit_restr(backFittingData, omegas[j], timePoints = timePoints,
-                                                      lengthAlphaGrid = lengthAlphaGrid, 
+                                                      lengthAlphaGrid = lengthAlphaGrid,
                                                       #alphaGrid = alphaGrid[[j]],
                                                       numReps = numReps)
       fittedValuesPerComponent[,j] <- getFittedValues(fittedFMMPerComponent[[j]])
@@ -183,7 +320,7 @@ fitFMM_unit_restr<-function(vData, omega, timePoints = seqTimes(length(vData)),
   # rest of the parameters.
   # step1FMM function is used to make this estimate
   grid <- expand.grid(alphaGrid, omega)
-  step1 <- usedApply(FUN = step1FMM, X = grid, vData = vData, timePoints = timePoints)
+  step1 <- usedApply(FUN = step1FMMOld, X = grid, vData = vData, timePoints = timePoints)
   colnames(step1) <- c("M","A","alpha","beta","omega","RSS")
 
   # We find the optimal initial parameters,
@@ -240,7 +377,7 @@ fitFMM_unit_restr<-function(vData, omega, timePoints = seqTimes(length(vData)),
 
     ## Step 1: initial parameters
     grid <- as.matrix(expand.grid(alphaGrid,omega))
-    step1 <- usedApply(FUN = step1FMM, X = grid, vData = vData, timePoints = timePoints)
+    step1 <- usedApply(FUN = step1FMMOld, X = grid, vData = vData, timePoints = timePoints)
     colnames(step1) <- c("M","A","alpha","beta","omega","RSS")
     previousBestPar <- bestPar
     bestPar <- bestStep1(vData,step1)
